from educore import pin, wifi, dht, oled, rfid, button, mqttclient, ultrasonic
import time
from easydisplay import EasyDisplay
from machine import Pin, SPI
from ili9341 import ILI9341

# 初始化传感器
dhtsensor = dht(1)  # 温湿度传感器
scan_rfid = rfid()  # RFID 读卡器
bt_a = button(button.a)  # 按键 A
p8 = pin(8)  #实例化 按钮引脚8
led = pin(12)
sonic = ultrasonic(trig=0, echo=0)  # 超声波传感器

# 容器参数
container_height = 30.0  # 容器总高度（单位：厘米）
container_area = 10.0  # 容器底面积（单位：平方厘米）

# Wi-Fi 连接
wifi.connect(ssid="BCM-RJ", psd="bcm123456", timeout=50000)

# 初始化显示屏
spi = SPI(1, baudrate=40000000, phase=0, polarity=0, sck=Pin(12), mosi=Pin(21))
lcd = ILI9341(spi, cs=Pin(14), dc=Pin(13), rst=Pin(0), w=320, h=240, r=2)
ed = EasyDisplay(lcd, "RGB565", font="/font/text_lite_16px_2312.v3.bmf", show=True, color=0xFFFF, clear=False, auto_wrap=True)

# 初始化状态
last_temp = None
last_hum = None
last_cow_id = None
last_milk_amount = None
send_status = '收集数据'

def on_button_pressed_a():
    print("Button A pressed!")

bt_a.event_pressed =on_button_pressed_a #绑定按键事件，当按键 A 被按下时回调 myfunction 函数。

def send_msg(topic, msg):
    mqttclient.connect(server="192.168.1.22", port=1883, user="siot", psd="dfrobot")
    mqttclient.publish(topic=topic, content='->' + msg)

while True:
    # 读取温湿度
    temp, hum = dhtsensor.read()
     
    # 读取 RFID
    rf = scan_rfid.scanning(wait=True)
    cow_id = rf.serial_number() if rf else "未识别"
  
    # 读取超声波传感器数据
    liquid_height = sonic.distance()  # 获取液面高度（单位：厘米）
    print(liquid_height)
    
    # 计算产奶量
    milk_amount = round((container_height - liquid_height) * container_area ,2) # 产奶量（单位：立方厘米）
    
    # 创建数据字典
    data = {
        "temperature": temp,
        "humidity": hum,
        "cow_id": cow_id,
        "milk_amount": milk_amount  # 计算后的产奶量
    }
    
    # 发送数据
    if bt_a.status() == 0 or p8.read_digital():  # 按键 A 被按下
        led.write_digital(1)
        send_status = '开始发送...'
        ed.text(f"运行状态: {send_status}           ", 0, 190, color=0x0000FF, size=16)
        send_msg('siot/temp', str(data["temperature"]))
        send_msg('siot/hum', str(data["humidity"]))
        send_msg(f'siot/cow{data["cow_id"]}', str(data["milk_amount"]))
        
        send_status = '发送完成'
        time.sleep(1)
        send_status = '收集数据'
        led.write_digital(0)
    # 显示信息
    #ed.clear()  # 清屏
    #ed.text("智能产奶监测系统", 20, 20, color=0x7E0, size=24)  # 系统标题
    ed.bmp("/img/banner.bmp", 0, 0)
    

    # 只更新变化的内容
    if last_temp != temp:
        ed.text(f"温度: {temp} 度", 10, 100, color=0x0000FF, size=16)
        last_temp = temp

    if last_hum != hum:
        ed.text(f"湿度: {hum}%", 130, 100, color=0x0000FF, size=16)
        last_hum = hum

    if last_cow_id != cow_id:
        ed.text(f"编号: {cow_id}", 10, 160, color=0x0000FF, size=16)
        last_cow_id = cow_id
        
    if last_milk_amount != milk_amount:
        ed.text(f"产量: {milk_amount} L", 10, 190, color=0x0000FF, size=16)
        last_milk_amount = milk_amount
    
    
    ed.text(f"状态: {send_status}           ", 10, 250, color=0x0000FF, size=16)
    
    # 显示当前时间
    current_time = time.localtime()
    time_str = "{:04}-{:02}-{:02} {:02}:{:02}:{:02}".format(current_time[0], current_time[1], current_time[2],
                                                           current_time[3], current_time[4], current_time[5])     
    ed.text(f"时间: {time_str}", 10, 280, color=0x0000FF, size=16)
    time.sleep(0.2) 